'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.winSpawn = exports.spawnShell = undefined;

var _child_process = require('child_process');

var _defaultShell = require('default-shell');

var _defaultShell2 = _interopRequireDefault(_defaultShell);

var _os = require('os');

var _net = require('net');

var _net2 = _interopRequireDefault(_net);

var _ramda = require('ramda');

var _ramda2 = _interopRequireDefault(_ramda);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _nodePty = require('node-pty');

var pty = _interopRequireWildcard(_nodePty);

var _tmp2 = require('tmp');

var _tmp3 = _interopRequireDefault(_tmp2);

var _promisifyNode = require('promisify-node');

var _promisifyNode2 = _interopRequireDefault(_promisifyNode);

var _constants = require('../constants');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var tmp = (0, _promisifyNode2.default)(_tmp3.default);
// Cleanup even when there are errors
tmp.setGracefulCleanup();

var IS_WINDOWS = process.platform === 'win32';

var buildSocketPath = function buildSocketPath() {
  return tmp.dir().then(function (tmpPath) {
    var prefix = IS_WINDOWS ? '\\\\.\\pipe' : '';
    var cleanedPath = IS_WINDOWS ? tmpPath.replace('C:\\', '') : tmpPath;
    cleanedPath = cleanedPath.replace('.tmp', '');
    return _path2.default.join(prefix, cleanedPath, 'echo.sock');
  });
};

var buildCredentialsCallbackProcess = function buildCredentialsCallbackProcess(spawnedProcess, callback, reject) {
  var credentialsCallback = function credentialsCallback(promptType) {
    return function (username, password, cancel) {
      if (cancel) {
        // we are done here
        spawnedProcess.destroy();
        return reject(new Error('LFS action cancelled'));
      }

      spawnedProcess.write(promptType === _constants.promptTypes.USERNAME ? username : password);
      spawnedProcess.write(_os.EOL);
    };
  };

  return function (chunk) {
    var output = chunk.toString().trim().toLowerCase();

    if (output.match(_constants.regex.USERNAME)) {
      callback(output, credentialsCallback(_constants.promptTypes.USERNAME));
    } else if (output.match(_constants.regex.PASSWORD)) {
      callback(output, credentialsCallback(_constants.promptTypes.PASSWORD));
    } else if (output.match(_constants.regex.PASSPHRASE)) {
      callback(output, credentialsCallback(_constants.promptTypes.PASSPHRASE));
    }
    return output;
  };
};

var buildSocket = function buildSocket(size, closeProcess, socketName, mainResolve, mainReject) {
  return new Promise(function (resolve, reject) {
    var bufferList = [];
    var currentSize = 0;
    var shouldReject = false; // in case an error is thrown but it is null or undefined
    var error = void 0;
    var rejectionHandler = void 0;

    var handleErrorWith = function handleErrorWith(_rejectionHandler, socket) {
      return function (_error) {
        error = _error;
        rejectionHandler = _rejectionHandler;
        shouldReject = true;

        if (socket) {
          socket.destroy();
        }
      };
    };

    var closed = function closed() {
      closeProcess();

      if (shouldReject) {
        rejectionHandler(error);
        return;
      }

      mainResolve({ stdout: Buffer.concat(bufferList) });
    };

    var socketServer = _net2.default.createServer(function (socket) {
      socket.on('close', closed);
      socket.on('data', function (data) {
        currentSize += data.length;
        bufferList.push(data);
        if (currentSize === size) {
          socket.destroy();
          socketServer.close(function () {});
        }
      });
      socket.on('error', handleErrorWith(mainReject, socket));
    });
    socketServer.listen(socketName);
    socketServer.on('listening', function () {
      resolve(socketName);
    });
    socketServer.on('error', handleErrorWith(reject));
    socketServer.on('close', closed);
  });
};

var spawnShell = exports.spawnShell = function spawnShell(command, opts, size, callback) {
  return new Promise(function (resolve, reject) {
    var spawnedProcess = void 0;
    var destroyProcess = function destroyProcess() {
      return spawnedProcess.destroy();
    };
    return buildSocketPath().then(function (socket) {
      return buildSocket(size, destroyProcess, socket, resolve, reject);
    }).then(function (socketName) {
      var options = _ramda2.default.mergeDeepRight(opts, { env: process.env, encoding: null });

      spawnedProcess = pty.spawn(_defaultShell2.default, ['-i'], options);

      var processChunk = callback && typeof callback === 'function' ? buildCredentialsCallbackProcess(spawnedProcess, callback, reject) : function (chunk) {
        return chunk.toString();
      };

      spawnedProcess.write(command + ' | nc -U ' + socketName);
      spawnedProcess.write(_os.EOL);

      spawnedProcess.on('data', function (data) {
        if (data.code === 'EIO') {
          // Do nothing, this is a node-pty error
          return;
        }

        processChunk(data);
      });

      spawnedProcess.on('error', function (err) {
        if (err.code === 'EIO') {
          // Do nothing, this is a node-pty issue
          return;
        }

        return reject(err);
      });
    }).catch(reject);
  });
};

var winSpawn = exports.winSpawn = function winSpawn(command, input, opts) {
  return new Promise(function (resolve, reject) {
    var options = _ramda2.default.mergeDeepRight(opts, { env: process.env, shell: true });

    var argList = command.trim().split(' ');
    var cmd = argList.shift();
    var args = argList;

    var spawnedProcess = (0, _child_process.spawn)(cmd, args, options);

    var bufferList = [];
    spawnedProcess.stdout.on('data', function (data) {
      if (data.code === 'EIO') {
        // Do nothing, this is a node-pty error
        return;
      }

      bufferList.push(data);
    });

    var closeOrExit = function closeOrExit() {
      var code = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;
      return resolve({
        code: code,
        stdout: Buffer.concat(bufferList)
      });
    };

    spawnedProcess.on('close', closeOrExit);
    spawnedProcess.on('exit', closeOrExit);
    spawnedProcess.stderr.on('data', function (data) {
      reject(new Error(data.toString()));
    });

    spawnedProcess.stdin.write(input);
    spawnedProcess.stdin.write(_os.EOL);
  });
};

var spawn = function spawn(command, opts, callback) {
  return new Promise(function (resolve, reject) {
    var options = _ramda2.default.mergeDeepRight(opts, { env: process.env });

    var argList = command.trim().split(' ');
    var cmd = argList.shift() + (IS_WINDOWS ? '.exe' : '');
    var args = argList;

    var stdout = '';
    var spawnedProcess = pty.spawn(cmd, args, options);

    var processChunk = callback && typeof callback === 'function' ? buildCredentialsCallbackProcess(spawnedProcess, callback, reject) : function (chunk) {
      return chunk.toString();
    };

    spawnedProcess.on('data', function (data) {
      if (data.code === 'EIO') {
        // Do nothing, this is a node-pty error
        return;
      }

      stdout += processChunk(data);
    });

    var shouldReject = false; // in case an error is thrown but it is null or undefined
    var error = void 0;
    spawnedProcess.on('error', function (_error) {
      if (_error.code === 'EIO') {
        // Do nothing, this is a node-pty error
        return;
      }

      error = _error;
      shouldReject = true;
    });
    spawnedProcess.on('exit', function () {
      var code = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : 0;

      // without this call, we will leave winpty-agents around.
      spawnedProcess.destroy();

      if (shouldReject) {
        reject(error);
        return;
      }

      resolve({
        code: code,
        stdout: stdout
      });
    });
  });
};

exports.default = spawn;